/*
 * ProxyCallbackHandler.java
 */

package org.grizzly.tunnel;

import com.sun.grizzly.CallbackHandler;
import com.sun.grizzly.ConnectorHandler;
import com.sun.grizzly.Context;
import com.sun.grizzly.Controller;
import com.sun.grizzly.IOEvent;
import com.sun.grizzly.util.OutputWriter;
import java.io.EOFException;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

/**
 * @author Alexey Stashok
 */
public class ProxyCallbackHandler implements CallbackHandler<Context> {
    private static final int BUFFER_SIZE = 2048;
    
    private ProxyFilter filter;
    private ConnectorHandler connectorHandler;
    private ByteBuffer buffer;
    
    public ProxyCallbackHandler(ProxyFilter filter,
            ConnectorHandler connectorHandler) {
        
        this.filter = filter;
        this.connectorHandler = connectorHandler;
        buffer = ByteBuffer.allocate(BUFFER_SIZE);
    }
    
    public void onConnect(IOEvent<Context> ioEvent) {
        SelectionKey key = ioEvent.attachment().getSelectionKey();
        connectorHandler.finishConnect(key);
        ioEvent.attachment().getController().registerKey(key,
                SelectionKey.OP_READ,
                Controller.Protocol.TCP);
    }
    
    public void onRead(IOEvent<Context> ioEvent) {
        Context context = ioEvent.attachment();
        SelectionKey key = context.getSelectionKey();
        Controller controller = context.getController();
        SocketChannel socketChannel = (SocketChannel)key.channel();
        
        try {
            SocketChannel serverChannel = filter.getServerChannel(connectorHandler);
            int nRead = 0;
            while((nRead = Utils.forceRead(buffer, socketChannel)) > 0) {
                OutputWriter.flushChannel(serverChannel, buffer);
            }
            
            if (nRead == -1) throw new EOFException("EOF!");
            
            controller.registerKey(key,
                    SelectionKey.OP_READ,
                    Controller.Protocol.TCP);
            
        } catch (IOException ex){
            filter.closeByClient(context, connectorHandler);
        }
        
    }
    
    public void onWrite(IOEvent<Context> ioEvent) {
        
    }
}
